// 本文件实现t_quest_db的data字段的序列化和反序列化

package main

import (
	"bytes"
	"encoding/gob"
)

type PlayerDoingQuestDbData struct {
	TaskId           int
	TaskDbId         int64
	AcceptTime       int32
	ProgressMap      map[int]string
	IsProgressFinish bool
}

type PlayerDoneQuestDbData struct {
	TaskId            int
	LastDoneTimestamp int32 // 上次完成任务的时间戳
	PeriodDoneTimes   int   // 任务周期内完成的次数(天/周)
}

// 这个结构存db, 对应t_quest_db的data字段
type PlayerAllQuestDbData struct {
	DoneTaskMap  map[int]*PlayerDoneQuestDbData
	DoingTaskMap map[int64]*PlayerDoingQuestDbData
}

// 把this编码为二进制，存入data字段
func (this *PlayerAllQuestDbData) ToDbBlob() ([]byte, error) {
	buf := bytes.NewBuffer([]byte{})
	enc := gob.NewEncoder(buf)
	if err := enc.Encode(this); err != nil {
		return nil, err
	}

	return buf.Bytes(), nil
}

// 解析data字段
func (this *PlayerAllQuestDbData) FromDbBlob(data []byte) error {
	if len(data) > 0 {
		dec := gob.NewDecoder(bytes.NewBuffer(data))
		if err := dec.Decode(this); err != nil {
			return err
		}
	}

	if this.DoneTaskMap == nil {
		this.DoneTaskMap = make(map[int]*PlayerDoneQuestDbData)
	}

	if this.DoingTaskMap == nil {
		this.DoingTaskMap = make(map[int64]*PlayerDoingQuestDbData)
	}

	return nil
}
