// 本文件实现UDF扩展

package main

/*
#cgo CFLAGS: -Iinclude
#include <mysql.h>
#include <string.h>
#include <stdlib.h>
*/
import "C"
import (
	"bytes"
	_ "encoding/gob"
	"encoding/json"
	"fmt"
	"strings"
	"unsafe"
)

func main() {}

func getUintPointerValue(pointer *uint32, offset int) *C.uint {
	return (*C.uint)(unsafe.Pointer(uintptr(unsafe.Pointer(pointer)) + uintptr(offset)*uintptr(unsafe.Sizeof(C.uint(0)))))
}

func getCharPointerValue(pointer **C.char, offset int) **C.char {
	var c C.char
	return (**C.char)(unsafe.Pointer(uintptr(unsafe.Pointer(pointer)) + uintptr(offset)*uintptr(unsafe.Sizeof(&c))))
}

func getUlongPointerValue(pointer *C.ulong, offset int) *C.ulong {
	return (*C.ulong)(unsafe.Pointer(uintptr(unsafe.Pointer(pointer)) + uintptr(offset)*uintptr(unsafe.Sizeof(C.ulong(0)))))
}

func getBytePointerValue(pointer *C.char, offset int) *C.char {
	return (*C.char)(unsafe.Pointer(uintptr(unsafe.Pointer(pointer)) + uintptr(offset)*uintptr(unsafe.Sizeof(C.char(0)))))
}

//export questblob_init
func questblob_init(init *C.UDF_INIT, args *C.UDF_ARGS, msg *C.char) C.my_bool {
	init.maybe_null = 1
	if args.arg_count < 2 {
		s := C.CString("UnExpected err: args count len < 1")
		C.strcpy(msg, s)
		C.free(unsafe.Pointer(s))
		return 1
	}
	*getUintPointerValue(args.arg_type, 0) = C.STRING_RESULT
	*getUintPointerValue(args.arg_type, 1) = C.STRING_RESULT
	return 0
}

//export questblob_deinit
func questblob_deinit(init *C.UDF_INIT) {
	C.free(unsafe.Pointer(init.ptr))
}

//export questblob
func questblob(init *C.UDF_INIT, args *C.UDF_ARGS, result *C.char, length *C.ulong, is_null *C.char, error *C.char) *C.char {
	*is_null = 1
	chars := getCharPointerValue(args.args, 0)
	arglen := getUlongPointerValue(args.lengths, 0)
	names := getCharPointerValue(args.args, 1)
	namelen := getUlongPointerValue(args.lengths, 1)
	rb := C.GoBytes(unsafe.Pointer(*chars), C.int(*arglen))
	name := C.GoStringN(*names, C.int(*namelen))
	nameInfos := strings.Split(name, ".")

	_ = nameInfos[0] // 数据库表名
	_ = nameInfos[1] // 数据库字段名，实际情况中可根据这两个string，决定构造那个对象
	errStringPrefix := fmt.Sprintf("[Error]table:'%s' column:'%s' ", nameInfos[0], nameInfos[1])

	data := &PlayerAllQuestDbData{}
	err := data.FromDbBlob(bytes.NewBuffer(rb).Bytes())
	if err != nil {
		*is_null = 0
		errString := errStringPrefix + err.Error()
		*length = C.ulong(len([]byte(errString)))
		errS := C.CString(errString)
		init.ptr = errS
		return errS
	}

	rb, err = json.Marshal(data)
	if err != nil {
		*is_null = 0
		errString := errStringPrefix + " Marshal error: " + err.Error()
		*length = C.ulong(len([]byte(errString)))
		errS := C.CString(errString)
		init.ptr = errS
		return errS
	}

	*is_null = 0
	*length = C.ulong(len(rb))
	s := C.CString(string(rb))
	init.ptr = s
	return s
}
